window.Gleam.initSearch([{"doc":"glebs","title":"glebs","content":"# glebs\n\n[![Package Version](https://img.shields.io/hexpm/v/glebs)](https://hex.pm/packages/glebs)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glebs/)\n\n```sh\ngleam add glebs\n```\n\nGlebs is an OAuth PKCE helper for Gleam in Browser. You can easily use this with\nlustre, see [lustre integration](docs/lustre.md) for more information.\n\n## Usage\n\n### Create an OAuth2ClientConfig\n\n```gleam\nimport glebs\n\nlet config =\n  glebs.OAuth2ClientConfig(\n    client_id: &quot;12345&quot;,\n    authorize_url: &quot;https://oauthserverexample.com/authorize&quot;,\n    token_url: &quot;https://oauthserverexample.com/token&quot;,\n    redirect_uri: &quot;https://example.com/redirect&quot;,\n    scope: &quot;&quot;,\n  )\n```\n\nYou would usually set the config values during build time, or allow user to\nspecify the configuration and store it in local storage.\n\n### Get an authorization url and redirect the user\n\nUsing the config get an authorization url based on the config and redirect the\nuser to the url:\n\n```gleam\nimport glebs/request\nimport gleam/javascript/promise\nimport plinth/browser/window\n\nrequest.create_authorization_request_url(config)\n|&gt; promise.map_try(fn(authorize_url) {\n  let curr_window = window.self()\n\n  let _ =\n    storage.local()\n    |&gt; result.map(storage.set_item(_, &quot;glebs_verifier&quot;, authorize_url.1))\n\n  let curr_window = window.self()\n  window.set_location(curr_window, uri.to_string(authorize_url.0))\n\n  Ok(Nil)\n})\n```\n\nThe `request.create_authorization_request_url` function returns a tuple of\n`#(Uri, String)` where the first element is the authorization url and the second\nelement is the code verifier. You should store the code verifier in session\nstorage to be able to exchange the authorization code for an access token.\n\n### Exchange the authorization code for an access token\n\n```gleam\nimport glebs/request\nimport gleam/javascript/promise\n\nrequest.get_access_token(config, verifier, code)\n|&gt; promise.map(fn(res) {\n  case res {\n    Ok(token) -&gt; {\n      // Do something with the token\n    }\n    Error(err) -&gt; {\n      // Handle error\n    }\n  }\n})\n```\n\nThe `request.get_access_token` function takes the config, the code verifier and\nthe authorization code as input and returns an\n`Promise(Result(glebs.TokenResponse), String)` if the request was successful.\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glebs&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\n## Todo\n\n- [ ] Get new token from refresh token if token has expired\n- [ ] Add better error handling\n","url":"index.html"},{"doc":"glebs","title":"OAuth2ClientConfig","content":"pub type OAuth2ClientConfig {\n  OAuth2ClientConfig(\n    client_id: String,\n    authorize_url: String,\n    token_url: String,\n    redirect_uri: String,\n    scope: String,\n  )\n}\n\nOAuth2ClientConfig(\n  client_id: String,\n  authorize_url: String,\n  token_url: String,\n  redirect_uri: String,\n  scope: String,\n)\n\n\nSynonyms:\nglebs.OAuth2ClientConfig\nglebs OAuth2ClientConfig","url":"glebs.html#OAuth2ClientConfig"},{"doc":"glebs","title":"TokenResponse","content":"pub type TokenResponse {\n  TokenResponse(\n    access_token: String,\n    token_type: String,\n    expires_in: Int,\n    refresh_token: String,\n  )\n}\n\nTokenResponse(\n  access_token: String,\n  token_type: String,\n  expires_in: Int,\n  refresh_token: String,\n)\n\n\nSynonyms:\nglebs.TokenResponse\nglebs TokenResponse","url":"glebs.html#TokenResponse"},{"doc":"glebs","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nglebs.main\nglebs main","url":"glebs.html#main"},{"doc":"glebs","title":"glebs","content":"","url":"glebs.html"},{"doc":"glebs/crypto","title":"get_random_string","content":"pub fn get_random_string(length: Int) -&gt; Result(String, String)\n\nSynonyms:\nglebs/crypto.get_random_string\nglebs/crypto get_random_string","url":"glebs/crypto.html#get_random_string"},{"doc":"glebs/crypto","title":"get_random_values","content":"pub fn get_random_values(\n  data: BitArray,\n) -&gt; Result(BitArray, String)\n\nSynonyms:\nglebs/crypto.get_random_values\nglebs/crypto get_random_values","url":"glebs/crypto.html#get_random_values"},{"doc":"glebs/crypto","title":"hash","content":"pub fn hash(data: String) -&gt; Promise(Result(BitArray, String))\n\nSynonyms:\nglebs/crypto.hash\nglebs/crypto hash","url":"glebs/crypto.html#hash"},{"doc":"glebs/crypto","title":"glebs/crypto","content":"","url":"glebs/crypto.html"},{"doc":"glebs/code","title":"create_code_verifier","content":"pub fn create_code_verifier() -&gt; Result(String, String)\n\nSynonyms:\nglebs/code.create_code_verifier\nglebs/code create_code_verifier","url":"glebs/code.html#create_code_verifier"},{"doc":"glebs/code","title":"get_code_challenge_from_verifier","content":"pub fn get_code_challenge_from_verifier(\n  verifier: String,\n) -&gt; Promise(Result(String, String))\n\nSynonyms:\nglebs/code.get_code_challenge_from_verifier\nglebs/code get_code_challenge_from_verifier","url":"glebs/code.html#get_code_challenge_from_verifier"},{"doc":"glebs/code","title":"glebs/code","content":"","url":"glebs/code.html"},{"doc":"glebs/request","title":"create_authorization_request_url","content":"pub fn create_authorization_request_url(\n  config: OAuth2ClientConfig,\n) -&gt; Promise(Result(#(Uri, String), String))\n Create an authorization request url from the given config\n\n You can redirect the user to the returned url to initiate the authorization\n flow. Once authorization is complete, the user will be redirected back to\n the redirect_uri in the config with the authorization code in the query\n parameter.\n\n You can then exchange the authorization code for an access token using the\n `get_access_token` function.\nSynonyms:\nglebs/request.create_authorization_request_url\nglebs/request create_authorization_request_url","url":"glebs/request.html#create_authorization_request_url"},{"doc":"glebs/request","title":"get_access_token","content":"pub fn get_access_token(\n  config: OAuth2ClientConfig,\n  verifier: String,\n  code: String,\n) -&gt; Promise(Result(TokenResponse, String))\n\nSynonyms:\nglebs/request.get_access_token\nglebs/request get_access_token","url":"glebs/request.html#get_access_token"},{"doc":"glebs/request","title":"glebs/request","content":"","url":"glebs/request.html"}]);